@@ -1042,19 +1042,23 @@ file_is_image (const gchar *name,
 }
 
 
-gboolean 
-image_is_jpeg (const char *name)
+static gboolean 
+image_is_type (const char *name,
+	       const char *type)
 {
-	const char *result;
+	const char *result = NULL;
 
 	if (eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE)) {
 		char *n1 = g_filename_to_utf8 (name, -1, 0, 0, 0);
-		char *n2 = g_utf8_strdown (n1, -1);
-		char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
-		result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
-		g_free (n3);
-		g_free (n2);
-		g_free (n1);
+		if (n1 != NULL) {
+			char *n2 = g_utf8_strdown (n1, -1);
+			char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
+			if (n3 != NULL)
+				result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
+			g_free (n3);
+			g_free (n2);
+			g_free (n1);
+		}
 	} else 
 		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
 	
@@ -1062,7 +1066,21 @@ image_is_jpeg (const char *name)
 	if (result == NULL)
 		return FALSE;
 
-	return (strcmp (result, "image/jpeg") == 0);
+	return (strcmp (result, type) == 0);
+}
+
+
+gboolean 
+image_is_jpeg (const char *name)
+{
+	return image_is_type (name, "image/jpeg");
+}
+
+
+gboolean 
+image_is_gif (const char *name)
+{
+	return image_is_type (name, "image/gif");
 }
 
 
@@ -1604,6 +1622,9 @@ new_path_from_uri (GnomeVFSURI *uri)
 	char *path;
 	char *unescaped_path;
 
+	if (uri == NULL)
+		return NULL;
+
 	path = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
 	unescaped_path = gnome_vfs_unescape_string (path, NULL);
 	g_free (path);
@@ -1621,6 +1642,7 @@ resolve_all_symlinks (const char  *text_uri,
 	GnomeVFSFileInfo *info;
 	const char       *p;
 	int               n_followed_symlinks = 0;
+	gboolean          first_level = TRUE;
 
 	*resolved_text_uri = NULL;
 
@@ -1656,6 +1678,7 @@ resolve_all_symlinks (const char  *text_uri,
 			   info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME) {
 			GnomeVFSURI *resolved_uri;
 			char        *tmp_resolved_text_uri = NULL;
+			char        *symbolic_link = NULL;
 
 			n_followed_symlinks++;
 			if (n_followed_symlinks > MAX_SYMLINKS_FOLLOWED) {
@@ -1664,7 +1687,13 @@ resolve_all_symlinks (const char  *text_uri,
 				goto out;
 			}
 
-			resolved_uri = gnome_vfs_uri_resolve_relative (new_uri, info->symlink_name);
+			if (first_level && (info->symlink_name[0] != '/')) 
+				symbolic_link = g_strconcat ("/", info->symlink_name);
+			else 
+				symbolic_link = g_strdup (info->symlink_name);
+			resolved_uri = gnome_vfs_uri_resolve_relative (new_uri, symbolic_link);
+			g_free (symbolic_link);
+
 			tmp_resolved_text_uri = new_path_from_uri (resolved_uri);
 			gnome_vfs_uri_unref (resolved_uri);
 
@@ -1680,6 +1709,7 @@ resolve_all_symlinks (const char  *text_uri,
 
 			p = my_text_uri;
 		}
+		first_level = FALSE;
 
 		gnome_vfs_uri_unref (new_uri); 
 	}
